import os
import inflection
from django_clite.helpers.logger import *
from django_clite.helpers import sanitized_string
from django_clite.helpers import rendered_file_template
from django_clite.helpers import FSHelper

BASE_DIR = os.path.dirname(os.path.abspath(__file__)).rsplit('/', 1)[0]

TEMPLATE_DIR = os.path.join(BASE_DIR, 'templates')

TEMPLATES = [f for f in os.listdir(TEMPLATE_DIR) if f.endswith('tpl')]


class ManagerHelper(FSHelper):

    def create(self, model, **kwargs):

        model = self.check_noun(model)
        model = sanitized_string(model)
        classname = inflection.camelize(model)

        filename = f"{model.lower()}.py"

        template = 'manager.tpl'
        template_import = 'manager-import.tpl'

        content = rendered_file_template(
            path=TEMPLATE_DIR,
            template=template,
            context={'classname': classname, 'model': model}
        )

        import_content = rendered_file_template(
            path=TEMPLATE_DIR,
            template=template_import,
            context={'classname': classname, 'model': model}
        )

        self.add_import(
            template=template_import,
            content=import_content
        )

        self.create_file(
            path=self.cwd,
            filename=filename,
            content=content
        )

        resource = f"{classname}Manager"
        log_success(DEFAULT_CREATE_MESSAGE.format(filename, resource))

    def delete(self, model, **kwargs):
        model = self.check_noun(model)
        classname = inflection.camelize(model)

        filename = f"{model}.py"
        template_import = 'manager-import.tpl'

        if self.default_destroy_file(
            model=model,
            templates_directory=TEMPLATE_DIR,
            template_import=template_import
        ):

            resource = f"{classname}Manager"
            log_success(DEFAULT_DELETE_MESSAGE.format(filename, resource))



# generator:helpers
from .admin import AdminHelper
from .fixture import FixtureHelper
from .form import FormHelper
from .manager import ManagerHelper
from .model import ModelHelper
from .serializer import SerializerHelper
from .template import TemplateHelper
from .test import TestHelper
from .view import ViewHelper
from .viewset import ViewSetHelper



# runner:helpers
from .runner import RunnerHelper
from .build import BuildHelper
from .docker import DockerHelper
from .environment import EnvironmentHelper
from .migration import MigrationHelper
from .server import ServerHelper
from .fixture import FixtureHelper



from click.testing import CliRunner
from django_clite.commands.generator import serializer


def test_generate_serializer():
    runner = CliRunner()
    with runner.isolated_filesystem():
        result = runner.invoke(serializer, ['test_serializer'])

    assert result.exit_code == 0
    assert result.output == '1'



# commands:generator
from .main import generate
from .main import admin
from .main import form
from .main import manager
from .main import model
from .main import resource
from .main import serializer
from .main import sql_view
from .main import template
from .main import test
from .main import view
from .main import viewset



from click.testing import CliRunner
from django_clite.commands.generator import resource


def test_generate_resource():
    runner = CliRunner()
    with runner.isolated_filesystem():
        result = runner.invoke(resource, ['test_admin'])

    assert result.exit_code == 0
    assert result.output == '1'



import os
import subprocess
from django_clite.helpers.logger import *
from django_clite.helpers import rendered_file_template
from .runner import RunnerHelper

BASE_DIR = os.path.dirname(os.path.abspath(__file__)).rsplit('/', 1)[0]

TEMPLATE_DIR = os.path.join(BASE_DIR, 'templates')

TEMPLATES = [f for f in os.listdir(TEMPLATE_DIR) if f.endswith('tpl')]

BUILD_COMMAND = ['docker-compose', 'build']

START_COMMAND = ['docker-compose', 'up']


class DockerHelper(RunnerHelper):

    def run(self, **kwargs):
        pass

    def __create(self, project, template, filename):

        try:
            os.chdir(self.cwd)

            content = rendered_file_template(
                path=TEMPLATE_DIR,
                template=template,
                context={'project': project}
            )
            self.create_file(
                content=content,
                filename=filename
            )
            log_success(f"Created {filename}")
        except FileNotFoundError:
            log_error(f"Unable to create {filename}")

    def create_compose(self, project):

            template = 'docker-compose.tpl'
            filename = 'docker-compose.yml'

            self.__create(project, template, filename)

    def create_dockerfile(self, project):

        template = 'dockerfile.tpl'
        filename = 'Dockerfile'

        self.__create(project, template, filename)
        self.__create(project, 'docker-entrypoint.tpl', 'docker-entrypoint.sh')

    def build(self, verbose=False):

        try:
            os.chdir(self.cwd)
        except FileNotFoundError:
            pass

        try:
            if verbose:
                BUILD_COMMAND.insert(1, '--verbose')
            return subprocess.check_output(BUILD_COMMAND)
        except subprocess.CalledProcessError as e:
            return None

    def start(self, verbose=False):

        try:
            os.chdir(self.cwd)
        except FileNotFoundError:
            pass

        try:
            if verbose:
                START_COMMAND.insert(1, '--verbose')
            return subprocess.check_output(START_COMMAND)
        except subprocess.CalledProcessError as e:
            return None



import os
from django_clite.helpers.logger import *
from django_clite.helpers import sanitized_string
from django_clite.helpers import rendered_file_template
from django_clite.helpers import FSHelper

BASE_DIR = os.path.dirname(os.path.abspath(__file__)).rsplit('/', 1)[0]

TEMPLATE_DIR = os.path.join(BASE_DIR, 'templates')

TEMPLATES = [f for f in os.listdir(TEMPLATE_DIR) if f.endswith('tpl')]


class TemplateHelper(FSHelper):

    def create(self, model, **kwargs):

        model = sanitized_string(model)

        template = 'template.tpl'
        filename = f"{model}.html"
        if kwargs.get('class_type'):
            filename = f"{model.lower()}_{kwargs.get('class_type')}.html"

        content = rendered_file_template(
            path=TEMPLATE_DIR,
            template=template,
            context={}
        )

        self.create_file(
            path=self.cwd,
            filename=filename,
            content=content
        )

        log_success(f"Successfully created {filename} template.")

    def delete(self, model, **kwargs):
        model = self.check_noun(model)

        filename = f"{model.lower()}.html"

        if self.destroy_file(filename=filename, **kwargs):
            log_success(f"Successfully deleted {filename} template.")



# helpers:file_system
import os
import inflection
import fileinput
import subprocess
from .parser import sanitized_string
from .templates import rendered_file_template
from django_clite.helpers.finders import get_app_name
from django_clite.helpers.finders import get_project_name
from django_clite.helpers.finders import walk_up
from .logger import *


PREVIOUS_WORKING_DIRECTORY = '..'


class FSHelper(object):
    """
    FS, File System Helper

    Use this helper to handle all things related to
    searching, creating, and deleting files.
    """

    def __init__(self, cwd, dry=False, force=False, default=False, verbose=False):
        """
        :param cwd: current working directory
        :param dry: specifies if file system should be changed
        :param force: specifies if files should be overridden
        """

        self.__dry = dry
        self.__force = force
        self.__project_name = None
        self.__cwd = cwd
        self.__default = default
        self.__verbose = verbose
        self.__project_directory = None
        self.__path_to_management_file = None
        self.__management_file = None
        self.__app = get_app_name(verbose=verbose)

        self.__settings_file = self.find_settings_file()

        self.find_project_files()

        os.chdir(self.__cwd)

    def check_noun(self, noun):
        """
        Checks whether a noun is plural or singular and gives
        the option to change the noun from plural to singular.
        """
        noun = noun.lower()

        if inflection.pluralize(noun) == noun:
            log_standard(DEFAULT_NOUN_NUMBER_WARNING)
            singular = inflection.singularize(noun)

            if click.confirm(
                    DEFAULT_NOUN_NUMBER_OPTION.format(noun, singular)
            ):
                noun = singular
        return noun

    ##################################
    # Class properties

    @property
    def app_name(self):
        return self.__app

    @property
    def directory(self):
        return self.__project_directory

    @property
    def management_file(self):
        return self.__management_file \
            if self.__management_file \
            else self.find_project_files()[-1]

    @property
    def settings_file(self):
        return self.__settings_file

    @property
    def is_dry(self):
        return self.__dry

    @property
    def is_force(self):
        return self.__force

    def get_cwd(self):
        return self.__cwd

    def set_cwd(self, path):
        self.__cwd = path

    def get_project(self):
        return self.__project_name \
            if self.__project_name \
            else get_project_name(
                management_file=self.management_file,
                find_first=True
            )

    def set_project(self, name):
        self.__project_name = name

    def get_verbose(self):
        return self.__verbose

    def set_verbose(self, verbose):
        self.__verbose = verbose

    cwd = property(fget=get_cwd, fset=set_cwd)

    project_name = property(fget=get_project, fset=set_project)

    verbose = property(fget=get_verbose, fset=set_verbose)

    ##################################
    # Writing to the file system

    def default_create(self, model, context, **kwargs):

        model = self.check_noun(model)
        model = sanitized_string(model)
        classname = inflection.camelize(model)
        context['classname'] = classname

        filename = f"{model.lower()}.py"

        template = kwargs.get('template')
        template_import = kwargs.get('template_import')
        templates_directory = kwargs.get('templates_directory')

        content = rendered_file_template(
            path=templates_directory,
            template=template,
            context=context
        )

        import_content = rendered_file_template(
            path=templates_directory,
            template=template_import,
            context=context
        )

        self.add_import(
            template=template_import,
            content=import_content
        )

        self.create_file(
            path=self.cwd,
            filename=filename,
            content=content
        )

        scope = kwargs.get('scope') if kwargs.get('scope') else ''
        resource = f"{classname}{scope}"
        log_success(DEFAULT_CREATE_MESSAGE.format(filename, resource))

    def create_package(self, project, package, **kwargs):
        """
        Create a package and any associated modules
        :param project: name of the project this package belongs to
        :param package: name of the package to be created
        :return: True if package is created. False otherwise
        """

        project = sanitized_string(project)
        package = sanitized_string(package)

        content = f'# {project}:{kwargs.get("app")}:{package}'

        os.mkdir(package)
        os.chdir(package)

        _ = self.create_file(
            path=os.getcwd(),
            filename='__init__.py',
            content=content
        )

        if self.__verbose:
            log_info(f"Created package {package} at {os.getcwd()}")

        os.chdir(PREVIOUS_WORKING_DIRECTORY)
        return _

    def create_file(self, content, filename, path=None):
        """
        :param content: content for created file
        :param filename: name for created file
        :param path: location of created file
        :return: True if file is created. False otherwise
        """

        if self.is_dry:
            self.__log_dry(
                path=path,
                filename=filename,
                content=content
            )
            return False

        path = path if path else self.__cwd

        try:
            os.chdir(path)
        except FileNotFoundError:
            os.makedirs(path)
            os.chdir(path)

        try:
            with open(filename, 'x') as file:
                file.write(content)
                file.write('\n')
            file.close()
            os.chdir(path)
        except FileExistsError:
            if self.__force or click.confirm(DEFAULT_OVERRIDE_WARNING.format(filename)):
                with open(filename, 'w') as file:
                    file.write(content)
                    file.write('\n')
                file.close()
                os.chdir(path)
            return False

        if self.verbose:
            log_info(f"Created {filename} at {path}")

        return True

    def destroy_file(self, filename, path=None):
        """
        :param filename: name of file to be destroyed
        :param path: location of file to be destroyed
        :return: True if file is created. False otherwise
        """

        path = path if path else self.__cwd

        if self.is_dry:
            click.echo(DEFAULT_DESTROY_LOG.format(filename, path))
            return False
        else:
            try:
                os.chdir(path)
                os.remove(filename)
            except FileNotFoundError:
                log_error(f"File {filename} does not exist.")
                return False

        if self.verbose:
            log_info(f"Removed {filename} from {path}")

        return True

    def default_destroy_file(self, model, **kwargs):

        if not self.__dry and not self.__force:
            click.confirm('Are you sure you want to delete this file?', abort=True)

        filename = f'{model}.py'
        classname = inflection.camelize(model)

        try:
            # Optional import removal
            content = rendered_file_template(
                path=kwargs.get('templates_directory'),
                template=kwargs.get('template_import'),
                context={
                    'classname': classname,
                    'model': model
                }
            )

            self.remove_import(content=content)
        except AttributeError or ValueError:
            pass

        return self.destroy_file(
            filename=filename,
            path=self.__cwd
        )

    def add_import(self, content, path=None, **kwargs):
        """
        :param content: content for import statement
        :param path: path to __init__ file
        :param kwargs:
        :return: template content
        """

        filename = '__init__.py'

        path = path if path else self.__cwd

        if self.is_dry:
            self.__log_dry(
                path=path,
                filename=filename,
                content=content,
            )
            return content
        try:
            os.chdir(path)

            # Prevent duplicate imports
            for line in fileinput.input(filename):
                if content in line:
                    log_error(DEFAULT_IMPORT_WARNING)
                    fileinput.close()
                    return
            fileinput.close()

            with open(filename, 'a') as file:
                file.write(content)
                file.write('\n')
            file.close()
        except FileNotFoundError:
            with open(filename, 'w') as file:
                file.write(content)
                file.write('\n')

        if self.verbose:
            log_info(f"Added: {content} to {path}/{filename}")

        return content

    def remove_import(self, content, path=None, **kwargs):
        """
        :param content: content for import statement
        :param path: path to __init__ file
        :param kwargs:
        :return: template content
        """

        filename = '__init__.py'

        path = path if path else self.__cwd

        if not self.is_dry:
            try:
                os.chdir(path)

                # Find import line to remove
                s = open(filename).read()
                s = s.replace(content, '')
                f = open(filename, 'w')
                f.write(s)
                f.close()

                for line in fileinput.FileInput(filename, inplace=1):
                    if line.rstrip():
                        print(line, end='')

                if self.verbose:
                    log_info(f"Removed: {content} from {path}/{filename}")

            except FileNotFoundError:
                raise click.Abort

    def parse_templates(self, parings, names, directory, folders=None, **kwargs):
        """
        :param parings: mapping of file and template names
        :param names: names of templates to be parsed
        :param directory: template files directory
        :param folders: project folders to create
        :param kwargs: context and other arguments
        :return: True if templates are parsed
        """

        if folders is not None:
            for folder in folders:
                try:
                    os.makedirs(folder)
                except FileExistsError:
                    log_error(f"Directory {folder} already exists")

        for file, template in parings.items():
            i = names.index(template)
            template = names[i]

            content = rendered_file_template(
                path=directory,
                template=template,
                context=kwargs.get('context')
            )

            self.create_file(
                path=os.getcwd(),
                filename=file,
                content=content,
            )

            if self.verbose:
                log_info(f"Created file {file} from {template}")

        return True

    def create_repository(self):
        if not self.is_dry:
            subprocess.check_output(['git', 'init'])
            subprocess.check_output(['git', 'add', '--all'])
            subprocess.check_output(['git', 'commit', '-m', 'Initial commit'])
            log_success('Successfully initialized git repository')
            return True
        log_success("Skipping creation of git repository")
        return False

    ##################################
    # Locate files

    def find_project_files(self, path=None):
        """
        Searches the set working directory for any of 4 key Django
        files to determine from where a command is being run:

            manage.py, settings.py, wsgi.py, and apps.py

        This is used to ensure commands that create or delete resources
        are always run in the correct directory and within a Django project.
        """

        current_dir = path if path else self.__cwd

        code = 0
        path = None

        levels = (
            ('manage.py', 1),  # <-- project
            ('settings.py', 2),  # <-- project/project
            ('wsgi.py', 2),  # <-- project/project
            ('apps.py', 3),  # <-- project/project/app
        )

        for filename, c in levels:
            if filename in os.listdir(current_dir):
                code = c
                path = current_dir

        if code == 1:
            self.__project_directory = path
            self.__management_file = os.path.join(
                self.__project_directory, 'manage.py'
            )

        elif code == 2 or code == 3:
            for c, d, f in walk_up('.'):
                if 'manage.py' in f:
                    self.__project_directory = c
                    self.__management_file = os.path.join(
                        self.__project_directory, 'manage.py'
                    )
                    break

        return path, self.__project_directory, self.__management_file

    def find_settings_file(self):
        """
        Finds settings.py file
        :return: file or None
        """
        settings = os.path.join(self.__cwd, 'settings.py')
        if not os.path.exists(settings):
            return None
        return settings

    def list_project_directory(self):
        for f in os.listdir(self.__cwd):
            log_standard(f)

    ##################################
    # Logging

    def __log_dry(self, content, filename, path=None):
        """
        Responsible for logging commands to stdout.
        Used when --dry flag is specified.
        """

        path = path if path else self.__cwd

        click.echo(DEFAULT_PARSED_CONTENT_LOG.format(
            filename,
            path,
            content
        ))

    ##################################

    def __str__(self):
        return 'FileSystemHelper'

    ##################################



# helpers:template
import jinja2


def rendered_file_template(path, template, context):
    """
    :param path: path to templates directory
    :param template: template to render
    :param context: context dict for template
    :return: rendered template
    """

    _ = jinja2.Environment(
        loader=jinja2.FileSystemLoader(path)
    ).get_template(template).render(context)

    return _



from click.testing import CliRunner
from django_clite.commands.generator import admin


def test_generate_admin():
    runner = CliRunner()
    with runner.isolated_filesystem():
        result = runner.invoke(admin, ['test_admin'])

    assert result.exit_code == 0



import os
import inflection
from django_clite.helpers.logger import *
from django_clite.helpers import sanitized_string
from django_clite.helpers import rendered_file_template
from django_clite.helpers import FSHelper

BASE_DIR = os.path.dirname(os.path.abspath(__file__)).rsplit('/', 1)[0]

TEMPLATE_DIR = os.path.join(BASE_DIR, 'templates')

TEMPLATES = [f for f in os.listdir(TEMPLATE_DIR) if f.endswith('tpl')]


class ViewHelper(FSHelper):

    def create(self, model, class_type, **kwargs):
        model = self.check_noun(model)
        model = sanitized_string(model)
        classname = inflection.camelize(model)

        filename = f"{model.lower()}.py"
        template = 'view-function.tpl'
        template_import = 'view-function-import.tpl'

        if class_type is not None:
            filename = f"{model.lower()}_{class_type}.py"
            template = 'view-class.tpl'
            template_import = 'view-class-import.tpl'

        content = rendered_file_template(
            path=TEMPLATE_DIR,
            template=template,
            context={
                'model': model,
                'classname': classname,
                'class_type': class_type
            }
        )

        import_content = rendered_file_template(
            path=TEMPLATE_DIR,
            template=template_import,
            context={
                'model': model,
                'classname': classname,
                'class_type': class_type,
            }
        )

        self.add_import(
            template=template_import,
            content=import_content
        )

        self.create_file(
            path=self.cwd,
            filename=filename,
            content=content
        )

        resource = f"{model}_view."
        if class_type:
            resource = f"{classname}{class_type.capitalize()}View."
        log_success(DEFAULT_CREATE_MESSAGE.format(filename, resource))

    def delete(self, model, class_type, **kwargs):
        model = self.check_noun(model)
        classname = inflection.camelize(model)

        filename = f"{model}.py"
        template_import = 'view-function-import.tpl'

        if class_type is not None:
            filename = f"{model}_{class_type}.py"
            template_import = 'view-class-import.tpl'

        content = rendered_file_template(
            path=TEMPLATE_DIR,
            template=template_import,
            context={
                'model': model,
                'classname': classname,
                'class_type': class_type,
            }
        )

        self.remove_import(content=content)

        if self.destroy_file(
            filename=filename,
            path=self.cwd
        ):

            resource = f"{model}_view."
            if class_type:
                resource = f"{classname}{class_type.capitalize()}View."
            log_success(DEFAULT_DELETE_MESSAGE.format(filename, resource))

        return True



# django-clite
from django_clite.commands.generator.helpers import AdminHelper
from django_clite.commands.generator.helpers import FixtureHelper
from django_clite.commands.generator.helpers import FormHelper
from django_clite.commands.generator.helpers import ManagerHelper
from django_clite.commands.generator.helpers import ModelHelper
from django_clite.commands.generator.helpers import SerializerHelper
from django_clite.commands.generator.helpers import TemplateHelper
from django_clite.commands.generator.helpers import TestHelper
from django_clite.commands.generator.helpers import ViewHelper
from django_clite.commands.generator.helpers import ViewSetHelper



import os
from django_clite.helpers import get_project_name
from django_clite.helpers import find_project_files
from django_clite.helpers.logger import *
from .helpers import *
from django_clite.commands.inspector.main import InspectorHelper
from django_clite.commands.inspector.main import apps as inspect_apps


def wrong_place_warning(ctx):
    if (ctx.obj['path'] and ctx.obj['project_name']) is None:
        log_error(DEFAULT_MANAGEMENT_ERROR)
        log_standard('')
        log_standard(DEFAULT_MANAGEMENT_ERROR_HELP)
        raise click.Abort


@click.group()
@click.pass_context
@click.option('--dry', is_flag=True, help="Display output without creating files.")
@click.option('--default', is_flag=True, help="Apply all default configurations.")
@click.option('--verbose', is_flag=True, help="Run in verbose mode.")
def create(ctx, dry, default, verbose):
    """
    Creates projects and apps.
    """
    ctx.ensure_object(dict)

    ctx.obj['dry'] = dry
    ctx.obj['default'] = default
    ctx.obj['verbose'] = verbose

    ctx.obj['helper'] = CreatorHelper(
        cwd=os.getcwd(),
        dry=dry,
        default=default,
        verbose=verbose
    )

    p, m, f = find_project_files(os.getcwd())  # project path, project directory, management_file

    ctx.obj['path'] = p
    ctx.obj['file'] = f
    ctx.obj['management'] = m
    ctx.obj['project_name'] = get_project_name(f)


@create.command()
@click.argument('name')
@click.option('--docker', is_flag=True, help="Add support for Docker.")
@click.option('--dokku', is_flag=True, help="Add support for Dokku.")
@click.option('-a', '--custom-auth', is_flag=True, help="Add support for custom AUTH_USER_MODEL.")
@click.argument('apps', nargs=-1)
@click.pass_context
def project(ctx, name, docker, dokku, custom_auth, apps):
    """
    Creates a new django project.

    This is similar to using `django-admin startproject project_name` but with some added functionality.
    The command can handle the creation of apps upon creation of any project, you can simply specify the name of
    the apps after the project name:

        D new project website app1 app2 app3 ...

    Another added functionality is the ability to setup Docker and Dokku.
    To do so, use the respective flags --docker and --dokku.

    Also helpful is the ability to initialize your application with a custom authentication model of your choosing.
    When the --custom-auth flag is specified, CLI will add an `authentication` app to your project
    along with a subclass of the AbstractUser model which you can extend for your own authentication purposes.
    """

    helper = ctx.obj['helper']

    helper.create_project(
        project=name,
        apps=apps,
        default=ctx.obj['default'],
        docker=docker,
        dokku=dokku,
        custom_auth=custom_auth,
    )


@create.command()
@click.argument('apps', nargs=-1)
@click.option('--is-auth', is_flag=True, help="Add User for custom authentication.")
@click.option('--project-name', '-p', help="Specify name of your project.")
@click.option('--directory', '-d', type=click.Path(), help="Specify path to your management.")
@click.option('--api', is_flag=True, help="Add a special api urls module to your app directory.")
@click.pass_context
def app(ctx, apps, is_auth, project_name, directory, api):
    """
    Creates new django apps.

    This is similar to using `django-admin startapp app_name`
    but adds more structure to your app by creating packages for forms, models,
    serializers, tests, templates, views, and viewsets.

    The command can accept multiple apps as arguments. So,

        D new app shop blog forum

    will work prompt the CLI to create all 4 apps two levels within your project's
    directory, i.e mysite/mysite. The CLI tries to identify where the management module for your
    project is so it can place your app files in the correct location. This helps with consistency
    as the CLI can infer module/package scopes when performing other automatic configurations.

    As part of the CLI convention, each app is assigned its own `urls.py` file, which can be used to route urls on a per app basis. Another convention the CLI adopts is
    to add a viewsets package to the app's directory by default (for use with DRF). Within the viewsets directory
    a DRF router is instantiated in `router.py` and its urls added to each app's urlpatterns by default.
    """

    # Get project name from arguments
    __project_name = project_name if project_name else ctx.obj['project_name']
    __directory_path = directory if directory else ctx.obj['management']

    try:
        path = os.path.join(__directory_path, __project_name)
    except TypeError:
        path = ''

    if not os.path.exists(path):
        log_error(DEFAULT_MANAGEMENT_ERROR)
        log_standard('')
        log_standard(DEFAULT_MANAGEMENT_ERROR_HELP)
        raise click.Abort()

    helper = CreatorHelper(
        cwd=path,
        dry=ctx.obj['dry'],
        default=ctx.obj['default'],
        verbose=ctx.obj['verbose']
    )

    auth_application = None

    if is_auth:
        if len(apps) == 1:
            auth_application = apps[0]
        else:
            auth_application = click.prompt(f"Which app will be used for authentication? {apps}")
            while auth_application not in apps:
                auth_application = click.prompt(f"Please choose a valid option: {apps}")

    for name in apps:
        is_custom_app = (name == auth_application)

        helper.create_app(
            project=__project_name,
            app=name,
            auth=is_custom_app,
            api=api,
        )


@create.command()
@click.option('--ignore-apps', is_flag=True, help="Do not add project apps to INSTALLED_APPS.")
@click.pass_context
def settings(ctx, ignore_apps):
    """
    Create settings file for project.
    """

    wrong_place_warning(ctx)

    project_name = ctx.obj['project_name']

    path = os.path.join(ctx.obj['management'], project_name)

    h = CreatorHelper(
        cwd=path,
        dry=ctx.obj['dry'],
        default=ctx.obj['default'],
        verbose=ctx.obj['verbose']
    )

    if not ignore_apps:
        ctx.obj['helper'] = InspectorHelper(cwd=ctx.obj['management'])
        apps = ctx.invoke(inspect_apps, no_stdout=True)
        h.create_settings(project=ctx.obj['project_name'], apps=apps)
    else:
        h.create_settings(project=ctx.obj['project_name'])



import os
import subprocess
from django_clite.helpers.logger import *
from django_clite.helpers import FSHelper
from django_clite.helpers import sanitized_string
from django_clite.helpers import rendered_file_template
from django_clite.commands.generator.helpers import (
    AdminHelper,
    FormHelper,
    ModelHelper,
    SerializerHelper,
    TestHelper,
    ViewSetHelper
)


BASE_DIR = os.path.dirname(os.path.abspath(__file__)).rsplit('/', 1)[0]

TEMPLATE_DIR = os.path.join(BASE_DIR, 'templates')

TEMPLATES = [f for f in os.listdir(TEMPLATE_DIR) if f.endswith('tpl')]

CURRENT_WORKING_DIRECTORY = '.'

PREVIOUS_WORKING_DIRECTORY = '..'

DEFAULT_APP_PACKAGES = {
    'admin', 'fixtures', 'forms',
    'middleware', 'models', 'serializers',
    'templates', 'views', 'viewsets'
}

DOKKU_TEMPLATES = {
    'app.json': 'app_json.tpl',
    'CHECKS': 'dokku_checks.tpl',
    'DOKKU_SCALE': 'dokku_scale.tpl',
    'Procfile': 'procfile.tpl',
}

TOP_LEVEL_TEMPLATES = {
    'CHECKS': 'dokku_checks.tpl',
    'Dockerfile': 'dockerfile.tpl',
    'docker-compose.yml': 'docker-compose.tpl',
    'docker-entrypoint.sh': 'docker-entrypoint.tpl',
    'Pipfile': 'Pipfile.tpl',
    'README.md': 'README.tpl',
    '.env': 'env.tpl',
    '.env-example': 'env.tpl',
    '.gitignore': 'gitignore.tpl',
}

INNER_LEVEL_TEMPLATES = {
    'settings_override.py': 'settings.tpl',
    'storage.py': 'storage.tpl',
}

UNWANTED_FILES = {
    'apps.py', 'admin.py', 'models.py',
    'tests.py', 'views.py', '__init__.py'
}

APP_TEMPLATES = {
    'urls.py': 'urls.tpl',
    'apps.py': 'apps.tpl'
}


class CreatorHelper(FSHelper):
    """
    Creator Helper

    Aids the creation of projects and apps.
    Class has access to file system through FSHelper.
    """

    def __init__(self, cwd, dry=False, force=False, default=False, verbose=False):
        super(CreatorHelper, self).__init__(cwd, dry, force, default, verbose)

    def create_project(self, project, apps, **kwargs):
        """
        Creates a project structure and its dependent packages
        """

        project = sanitized_string(project)

        # Generating project directory
        # with the help of `django-admin`
        if not self.is_dry:
            try:
                log_standard(f"Attempting to create project: {project}")
                subprocess.check_output([
                    'django-admin',
                    'startproject',
                    project
                ])
            except subprocess.CalledProcessError:
                log_error(DEFAULT_ERRORS['project'])
                raise click.Abort

        # Customize django project directory
        os.chdir(project)

        # Create top-level directories and top-level files
        folders = ['dokku', 'dummy']

        if kwargs.get('default') or kwargs.get('dokku'):
            self.parse_templates(
                parings=TOP_LEVEL_TEMPLATES,
                names=TEMPLATES,
                directory=TEMPLATE_DIR,
                folders=folders,
                context={
                    'project': project,
                    'timeout': 60,
                    'wait': 20,
                    'web': 1,
                }
            )

            # Customize Dokku configuration
            os.chdir('dokku')
            self.parse_templates(
                parings=DOKKU_TEMPLATES,
                names=TEMPLATES,
                directory=TEMPLATE_DIR,
                context={
                    'project': project,
                    'timeout': 60,
                    'wait': 20,
                    'web': 1,
                }
            )
            os.chdir(PREVIOUS_WORKING_DIRECTORY)

        # Customize project package
        os.chdir(project)
        self.parse_templates(
            parings=INNER_LEVEL_TEMPLATES,
            names=TEMPLATES,
            directory=TEMPLATE_DIR,
            context={
                'project': project,
                'apps': apps,
            }
        )

        # Create project apps inside the inner package
        if apps is not None:
            for app in apps:
                self.create_app(project=project, app=app)

        # Create optional `custom-auth`
        auth_app = 'authentication'
        if kwargs.get('custom_auth'):
            if self.create_app(
                project=project,
                app=auth_app,
                auth=True
            ):

                auth_app_path = os.getcwd()
                log_error(auth_app_path)

                log_error("Cannot create authentication app.")
                log_error(os.getcwd())

                # Handle authentication app
                os.chdir(auth_app)
                self.__handle_custom_auth(project=project, **kwargs)
                os.chdir(PREVIOUS_WORKING_DIRECTORY)

        # cd ../../
        os.chdir(PREVIOUS_WORKING_DIRECTORY)
        os.chdir(PREVIOUS_WORKING_DIRECTORY)

        # Create git repository
        self.create_repository()

    def create_app_package(self, package, project, app):
        """
        Create an app package and any associated sub-packages or modules
        """

        package = sanitized_string(package)
        app = sanitized_string(app)

        # Create app package
        self.create_package(
            project=project,
            package=package,
            app=app
        )

        # Package customizations
        os.chdir(package)

        if self.verbose:
            log_info(f"Performing customizations for {package}")

        self.create_package(project=project, package='helpers', app=app)
        if package == 'admin':
            self.create_package(project=project, package='actions', app=app)
            self.create_package(project=project, package='inlines', app=app)
            self.create_package(project=project, package='permissions', app=app)
        if package == 'models':
            self.create_package(project=project, package='managers', app=app)
            self.create_package(project=project, package='signals', app=app)
            self.create_package(project=project, package='tests', app=app)
            self.create_package(project=project, package='validators', app=app)
            self.create_package(project=project, package='permissions', app=app)
        if package == 'serializers':
            self.create_package(project=project, package='tests', app=app)
        if package == 'templates':
            self.create_package(project=project, package='tags', app=app)
        if package == 'viewsets':
            self.create_package(project=project, package='permissions', app=app)
            filename = 'router.py'
            index = TEMPLATES.index('router.tpl')
            template = TEMPLATES[index]

            content = rendered_file_template(
                path=TEMPLATE_DIR,
                template=template,
                context={'project': project, 'app': app}
            )

            self.create_file(
                path=os.getcwd(),
                filename=filename,
                content=content,
            )

        os.chdir(PREVIOUS_WORKING_DIRECTORY)

        if self.verbose:
            log_info(f"Finished customizing {package}")

    def create_app(self, project, app, auth=False, **kwargs):
        """
        :param project: name of django project
        :param app: name of django app
        :param auth: app is used for authentication
        :param kwargs:
        :return: True if app is created. False otherwise
        """

        project = sanitized_string(project)
        app = sanitized_string(app)

        if self.verbose:
            log_info(f"Creating {app} at {os.getcwd()}")

        # Create django application
        try:
            subprocess.check_output([
                'django-admin',
                'startapp',
                app
            ])
        except subprocess.CalledProcessError:
            log_error(DEFAULT_ERRORS['app'].format(app))
            return False

        # Perform app customizations
        os.chdir(app)

        if self.verbose:
            log_info(f"Performing customizations for {app}")

        # Remove unwanted files
        try:
            for unwanted in UNWANTED_FILES:
                os.remove(unwanted)
        except FileNotFoundError:
            pass

        if self.verbose:
            log_standard(f"Removed unwanted modules {UNWANTED_FILES}")

        # Parse templates for apps.py and urls.py
        try:
            for filename, template in APP_TEMPLATES.items():
                content = rendered_file_template(
                    path=TEMPLATE_DIR,
                    template=template,
                    context={'project': project, 'app': app}
                )

                self.create_file(
                    path=os.getcwd(),
                    filename=filename,
                    content=content
                )
        except OSError:
            pass

        # Create app-specific packages and module
        if kwargs.get('api'):
            DEFAULT_APP_PACKAGES.remove('admin')
            DEFAULT_APP_PACKAGES.remove('forms')
            DEFAULT_APP_PACKAGES.remove('templates')
            DEFAULT_APP_PACKAGES.remove('views')

        for package in DEFAULT_APP_PACKAGES:
            try:
                self.create_app_package(
                    package=package,
                    project=project,
                    app=app
                )
            except OSError as e:
                log_error(f"Cannot create package: {package}\n{e}")

        if auth:
            if self.verbose:
                log_info(f"Adding default User model for {app} app")

            self.__handle_custom_auth(project=project, app=app)

        os.chdir(PREVIOUS_WORKING_DIRECTORY)
        log_success(DEFAULT_APP_CREATION_LOG.format(app))

        return True

    def __handle_custom_auth(self, **kwargs):

        project = kwargs.get('project')
        app = kwargs.get('app')

        base = os.getcwd()

        paths = {
            'admin': f'{base}/admin',
            'forms': f'{base}/forms',
            'models': f'{base}/models',
            'serializers': f'{base}/serializers',
            'models_test': f'{base}/models/tests',
            'serializers_test': f'{base}/serializers/tests',
            'viewsets': f'{base}/viewsets'
        }

        AdminHelper(cwd=paths['admin']).create_auth_user()
        FormHelper(cwd=paths['forms']).create_auth_user()
        ModelHelper(cwd=paths['models']).create_auth_user()
        SerializerHelper(cwd=paths['serializers']).create_auth_user()
        ViewSetHelper(cwd=paths['viewsets']).create(model='user')

        TestHelper(cwd=paths['models_test']).create_auth_user(scope='model')
        TestHelper(cwd=paths['serializers_test']).create_auth_user(scope='serializer')

    def create_settings(self, project, apps=None):
        template = 'settings.tpl'
        filename = 'settings_override.py'

        content = rendered_file_template(
            path=TEMPLATE_DIR,
            template=template,
            context={'project': project, 'apps': apps}
        )

        return self.create_file(
            content=content,
            filename=filename,
        )



import os
import inflection
from django_clite.helpers.logger import *
from django_clite.helpers import sanitized_string
from django_clite.helpers import FieldParser

BASE_DIR = os.path.dirname(os.path.abspath(__file__)).rsplit('/', 1)[0]

TEMPLATE_DIR = os.path.join(BASE_DIR, 'templates')

TEMPLATES = [f for f in os.listdir(TEMPLATE_DIR) if f.endswith('tpl')]


class ModelHelper(FieldParser):

    def create(self, model, is_sql=False, **kwargs):
        model = sanitized_string(model)
        template = 'model.tpl' \
            if not kwargs.get('template') \
            else kwargs.get('template')
        template_import = 'model-import.tpl'

        # Get name of the parent class for this model (if any)
        base_model = self.check_noun(kwargs['inherits']) if kwargs['inherits'] else None
        scope = kwargs.get('scope', '')

        self.project_name = kwargs.get('project') if kwargs.get('project') else self.project_name

        # Check if model is in the `special` category before
        # attempting to create a module for it if one is not found.
        if base_model is not None:
            if not self.append_special_import(base_model, scope=scope):
                self.find_resource_in_scope(base_model)

        # Get database name
        table_name = f"{self.app_name}_{inflection.pluralize(model)}"

        # Parse model fields by name and type
        if kwargs.get('fields') is not None:
            self.parse_fields(model, kwargs.get('fields'))

        # Handle SQL views
        # if is_sql:
        #     template = 'sql-model.tpl'

        self.default_create(
            model,
            templates_directory=TEMPLATE_DIR,
            template=template,
            template_import=template_import,
            context={
                'api': kwargs.get('api'),
                'abstract': kwargs.get('abstract'),
                'base': self.special_import,
                'fields': self.fields_list,
                'imports': self.imports_list,
                'db_table': table_name,
                'model': model,
                'model_plural': inflection.pluralize(model),
                'is_managed': kwargs.get('is_managed'),
            }
        )

        # Ensure related models are created
        self.handle_dependencies(app=self.app_name, **kwargs)

        return self.admin_fields_list

    def delete(self, model, **kwargs):
        model = self.check_noun(model)
        classname = inflection.camelize(model)

        filename = f"{model}.py"
        template_import = 'model-import.tpl'

        if self.default_destroy_file(
            model=model,
            templates_directory=TEMPLATE_DIR,
            template_import=template_import
        ):

            resource = f"{classname}"
            log_success(DEFAULT_DELETE_MESSAGE.format(filename, resource))

    def create_auth_user(self):
        self.create(
            model='user',
            inherits=None,
            fields=None,
            template='model-user.tpl'
        )



import click
from django_clite.commands import create
from django_clite.commands import destroy
from django_clite.commands import generate
from django_clite.commands import run
from django_clite.commands import inspect


__author__ = 'Leo Neto'


class AliasedGroup(click.Group):
    """
    Adds support for abbreviated commands
    """
    def get_command(self, ctx, cmd_name):
        rv = click.Group.get_command(self, ctx, cmd_name)
        if rv is not None:
            return rv
        matches = [x for x in self.list_commands(ctx)
                   if x.startswith(cmd_name)]
        if not matches:
            return None
        elif len(matches) == 1:
            return click.Group.get_command(self, ctx, matches[0])
        ctx.fail('Too many matches: %s' % ', '.join(sorted(matches)))


@click.command(cls=AliasedGroup)
@click.version_option()
@click.pass_context
def main(ctx):
    """
    django-clite: by Leo Neto

    A CLI to handle the creation and management of your Django projects.

    The CLI has some options about how your project should be structured in order for it to maximize the
    amount of automatic configuration it can provide you. Since Django itself adheres to configuration over
    convention, you are free to bypass conventions of the CLI if you so choose.
    """

    # Note for contributors:
    #
    # Commands should be added as sub-commands of the main click group.
    # This enables sub-command chaining. For example:
    #       Add `run` command to `main`:
    #
    #       main.add_command(run)
    #
    #       Then, run:
    #       django_clite run run-sub-command`

    ctx.ensure_object(dict)

    # Note for contributors:
    #
    # File system helper included in cli context.
    # Helper needs to be passed down to sub-commands.


main.add_command(create)
main.add_command(destroy)
main.add_command(generate)
main.add_command(run)
main.add_command(inspect)

if __name__ == '__main__':
    main()



# helpers
from .logger import *
from .field_parser import FieldParser
from .parser import sanitized_string
from .templates import rendered_file_template
from .fs import FSHelper
from .finders import (
    find_project_files,
    find_settings_file,
    get_app_name,
    get_project_name,
    save_to_settings,
    replace_line,
    walk_up
)



import os
from .helpers import InspectorHelper
from django_clite.helpers import get_project_name
from django_clite.helpers import find_project_files
from django_clite.helpers.logger import *


def wrong_place_warning(ctx):
    if (ctx.obj['path'] and ctx.obj['project_name']) is None:
        log_error(DEFAULT_MANAGEMENT_ERROR)
        raise click.Abort


@click.group()
@click.pass_context
def inspect(ctx):
    """
    Inspect your Django project.
    """

    ctx.ensure_object(dict)

    p, m, f = find_project_files(os.getcwd())

    ctx.obj['path'] = p
    ctx.obj['file'] = f
    ctx.obj['management'] = m
    ctx.obj['project_name'] = get_project_name(f)

    wrong_place_warning(ctx)

    ctx.obj['helper'] = InspectorHelper(cwd=m)


@inspect.command()
@click.option('--paths', is_flag=True, help="Show app paths.")
@click.option('--no-stdout', is_flag=True, help="Do not print to stdout.")
@click.pass_context
def apps(ctx, paths, no_stdout):
    """
    Show your project apps.
    """

    return ctx.obj['helper'].get_apps(
        show_paths=paths,
        no_stdout=no_stdout
    )


@inspect.command()
@click.pass_context
def models(ctx):
    """
    Show your project models.
    """

    return ctx.obj['helper'].get_models()



import os
from django_clite.helpers import FSHelper
from django_clite.helpers.logger import log_info, log_standard


class InspectorHelper(FSHelper):

    def get_apps(self, show_paths=False, no_stdout=False):
        """
        Traverse file system in search for AppConfig files.

        :param show_paths: Toggle showing the path to each app
        :param no_stdout: Toggle printing to stdout
        :return: dictionary of app names and paths
        """

        current_apps = {
            c.rsplit('/', 1)[-1]: c
            for c, _, files in os.walk(self.cwd) if 'apps.py' in files
        }

        for app, path in sorted(current_apps.items()):

            if not no_stdout:
                log_info(app)

            if show_paths:
                if not no_stdout:
                    log_standard(f"{path}")

        return current_apps

    def get_app_paths(self):
        """
        Traverse file system in search for apps and their paths.

        :return: list of app paths
        """

        current_paths = sorted([
            c
            for c, _, files in os.walk(self.cwd)
            if 'apps.py' in files
        ])

        [log_info(path) for path in current_paths]

        return current_paths

    def get_models(self):
        """
        Retrieve models under each app's models package.

        :return:
        """

        models = []

        excluded_dirs = [
            'signals', 'tests', 'helpers',
            'validators', 'managers', '__pycache__'
        ]

        excluded_files = [
            "__init__.py"
        ]

        current_apps = {
            c.rsplit('/', 1)[-1]: c
            for c, _, files in os.walk(self.cwd) if 'apps.py' in files
        }

        for app, path in sorted(current_apps.items()):

            models_directory = path + "/models"

            log_info(app)

            for root, dirs, files in os.walk(models_directory):
                dirs[:] = [d for d in dirs if d not in excluded_dirs]
                files[:] = [f for f in files if f not in excluded_files]

                for f in files:
                    models.append({app: f})
                    log_standard(f)

            log_standard("---")

        return models

    def parse_model_attributes(self):
        pass



import os
import inflection
from django_clite.helpers.logger import *
from django_clite.helpers import FSHelper

BASE_DIR = os.path.dirname(os.path.abspath(__file__)).rsplit('/', 1)[0]

TEMPLATE_DIR = os.path.join(BASE_DIR, 'templates')

TEMPLATES = [f for f in os.listdir(TEMPLATE_DIR) if f.endswith('tpl')]


class SerializerHelper(FSHelper):

    def create(self, model, **kwargs):

        template = 'serializer.tpl' \
            if not kwargs.get('template') \
            else kwargs.get('template')
        template_import = 'serializer-import.tpl' \
            if not kwargs.get('template_import') \
            else kwargs.get('template_import')

        self.default_create(
            model,
            templates_directory=TEMPLATE_DIR,
            template=template,
            template_import=template_import,
            scope='Serializer',
            context={'model': model}
        )

    def delete(self, model):
        model = self.check_noun(model)
        classname = inflection.camelize(model)

        filename = f"{model}.py"
        template_import = 'serializer-import.tpl'

        if self.default_destroy_file(
            model=model,
            templates_directory=TEMPLATE_DIR,
            template_import=template_import
        ):

            resource = f"{classname}Serializer"
            log_success(DEFAULT_DELETE_MESSAGE.format(filename, resource))

    def create_auth_user(self):
        self.create(
            model='User',
            template='serializer-user.tpl'
        )



import os
from .runner import RunnerHelper


DEFAULT_LOAD_FIXTURES_COMMAND = ['python3', 'manage.py', 'loaddata']


class FixtureHelper(RunnerHelper):
  
    @classmethod
    def load_one(cls, path, fixture):
        os.chdir(path)

    @classmethod
    def load_all(cls, path):
        os.chdir(path)

    def run(self, **kwargs):
        pass



import os
import click
import inflection
import fileinput
from jinja2 import Template
from .logger import log_error
from .fs import FSHelper
from .parser import sanitized_string
from .templates import rendered_file_template
from faker import Faker
from faker.providers import company, date_time, internet, misc

fake = Faker()
fake.add_provider(company)
fake.add_provider(date_time)
fake.add_provider(internet)
fake.add_provider(misc)

DEFAULT_NOT_IN_SCOPE_WARNING = """Cannot find model {} in {} scope. Want to create model?"""

# Abbreviation, Django field type
DEFAULT_MODEL_FIELDS = {
    'belongsto': "ForeignKey",
    'big': 'BigIntegerField',
    'bigint': 'BigIntegerField',
    'bool': 'BooleanField',
    'boolean': 'BooleanField',
    'char': 'CharField',
    'date': 'DateField',
    'datetime': 'DateTimeField',
    'dec': 'DecimalField',
    'decimal': 'DecimalField',
    'duration': 'DurationField',
    'email': 'EmailField',
    'file': "FileField",
    'filepath': 'FilePathField',
    'fk': "ForeignKey",
    'foreignkey': "ForeignKey",
    'float': 'FloatField',
    'hasone': "OneToOneField",
    'hasmany': 'ManyToManyField',
    'image': "ImageField",
    'int': 'IntegerField',
    'integer': 'IntegerField',
    'ip': 'GenericIPAddressField',
    'ipaddress': 'GenericIPAddressField',
    'many': 'ManyToManyField',
    'manytomany': 'ManyToManyField',
    'one': "OneToOneField",
    'onetoone': "OneToOneField",
    'photo': "ImageField",
    'slug': 'SlugField',
    'string': 'TextField',
    'text': 'TextField',
    'time': 'TimeField',
    'url': 'URLField',
    'uuid': 'UUIDField',
}

# Django field type, default value
DEFAULT_MODEL_OPTIONS = {
    'BigIntegerField': '',
    'BooleanField': 'default=False',
    'CharField': 'max_length=100',
    'DateField': 'auto_now=True',
    'DateTimeField': 'auto_now=True',
    'DecimalField': '',
    'DurationField': '',
    'EmailField': '',
    'GenericIPAddressField': '',
    'FileField': "blank=True, upload_to='uploads/{}/'",
    'FilePathField': '',
    'FloatField': '',
    'ForeignKey': "{}, related_name='{}', on_delete=models.PROTECT",
    'ImageField': "blank=True, upload_to='uploads/{}/'",
    'IntegerField': '',
    'ManyToManyField': '{}, blank=True',
    'OneToOneField': "{}, related_name='{}', on_delete=models.CASCADE",
    'SlugField': 'unique=True',
    'TextField': 'blank=True',
    'TimeField': 'auto_now=True',
    'URLField': '',
    'UUIDField': 'default=uuid.uuid4, editable=False',
}

JSON_COMPATIBLE_FIELDS = {
    'BigIntegerField': 'pyint',
    'BooleanField': 'pybool',
    'CharField': 'text',
    'DateField': 'future_date',
    'DateTimeField': 'iso8601',
    'DecimalField': 'pydecimal',
    'EmailField': 'safe_email',
    'GenericIPAddressField': 'ipv4',
    'FileField': 'file_path',
    'FilePathField': 'file_path',
    'FloatField': 'pyfloat',
    'ImageField': 'image_url',
    'IntegerField': 'pyint',
    'SlugField': 'slug',
    'TextField': 'text',
    'TimeField': 'time',
    'URLField': 'url',
    'UUIDField': 'uuid4',
}

DEFAULT_SPECIAL_INHERITABLE_TYPES = {
    'abstract-user': 'django.contrib.auth.models',
    'abstract-base-user': 'django.contrib.auth.models',
    'base-user': 'django.contrib.auth.models',
    'user': 'django.contrib.auth.models',
}

UNSUPPORTED_ADMIN_FIELD_TYPES = {
    'FileField',
    'FilePathField',
    'ImageField',
    'ManyToManyField',
    'TextField'
}

BASE_DIR = os.path.dirname(os.path.abspath(__file__)).rsplit('/', 1)[0]

TEMPLATE_DIR = os.path.join(BASE_DIR, 'commands/generator/templates')

TEMPLATES = [f for f in os.listdir(TEMPLATE_DIR) if f.endswith('tpl')]


class FieldParser(FSHelper):
    # Field names and types
    fixture_fields = []

    # List of fields to be rendered in model template
    fields_list = []

    # List models referred to in ForeignKey, OneToOne, or ManyToMany fields.
    dependencies_list = []

    # List of imports to be rendered in model template
    imports_list = []

    # List of special imports to be rendered in model template
    special_import = []

    # List of fields for admin model
    admin_fields_list = []

    ####################################

    def __handle_tokens(self, field):
        """
        Should parse input attribute into individual tokens.
        Expect input to be of the form (char:name, text:bio, image:artwork)

        :param field: model field to be parsed
        :return: (field name, Django field type) or None
        """

        # i.e char:first_name
        parsed_token = field.split(":")

        # i.e char
        parsed_token_type = parsed_token[0]

        # i.e first_name
        parsed_token_name = parsed_token[1]

        # Ensure token exists and has minimum required length of 2
        for part in parsed_token:
            if len(part) < 2:
                return None

        # Check if parsed_token_type is a relationship field
        for abbreviation, field in DEFAULT_MODEL_FIELDS.items():
            if parsed_token_type.lower() == abbreviation.lower():
                return sanitized_string(parsed_token_name), DEFAULT_MODEL_FIELDS[abbreviation]

        return None

    def __handle_fixture(self, field_name, field_type):
        if field_type in JSON_COMPATIBLE_FIELDS:
            fixture_template = Template('''"{{ field_name }}": {{ field_value }},''')
            value = getattr(fake, JSON_COMPATIBLE_FIELDS[field_type])()

            if type(value).__name__ == 'str':
                value = f'"{value}"'

            if type(value).__name__ == 'bool':
                value = f'{value}'.lower()

            content = fixture_template.render(
                field_name=field_name,
                field_value=value
            )
            self.__append_fixture_field(content)

    def __handle_field(self, model, field_name, field_type):
        """
        Determines what options are present with each model field.

        :param model: the current model
        :param field_name: name of model field. i.e. first_name
        :param field_type: type of model field. i.e. CharField
        :return:
        """

        # Parse field type and name for fixtures
        self.__handle_fixture(field_name, field_type)

        # Parse field type and name for model
        if DEFAULT_MODEL_OPTIONS[field_type] is not None:
            template = 'model-field.tpl'

            if field_type == "ForeignKey":
                options = DEFAULT_MODEL_OPTIONS[field_type].format(
                    field_name.capitalize(),
                    inflection.pluralize(model.lower())
                )
                self.find_resource_in_scope(field_name)
                self.append_import(field_name)

            elif field_type == "OneToOneField":
                options = DEFAULT_MODEL_OPTIONS[field_type].format(
                    field_name.capitalize(),
                    inflection.singularize(model)
                )
                self.find_resource_in_scope(field_name)
                self.append_import(field_name)

            elif field_type == "ManyToManyField":
                options = DEFAULT_MODEL_OPTIONS[field_type].format(
                    field_name.capitalize()
                )
                self.find_resource_in_scope(field_name)
                self.append_import(field_name)

            elif field_type == "FileField" or "ImageField":
                options = DEFAULT_MODEL_OPTIONS[field_type].format(inflection.pluralize(field_name))

            else:
                options = DEFAULT_MODEL_OPTIONS[field_type]

                # Save model for admin list
                log_error(field_name)
                log_error(field_type)

            if field_type not in UNSUPPORTED_ADMIN_FIELD_TYPES:
                self.__append_admin_field(field_name)

            return rendered_file_template(
                path=TEMPLATE_DIR,
                template=template,
                context={
                    'name': field_name,
                    'type': field_type,
                    'options': options,
                    'lazy_name': field_name.replace("_", " "),
                }
            )

    def __append_admin_field(self, value):
        # TODO: Add default `id` field
        if value not in self.admin_fields_list:
            self.admin_fields_list.append(value)

    def __append_fixture_field(self, value):
        """Ensure last key does not have an added comma"""
        if value not in self.fixture_fields:
            if len(self.fixture_fields) > 0:
                self.fixture_fields[-1] += ','
            value = value.replace(',', '')
            self.fixture_fields.append(value)

    def parse_fields(self, model, fields):
        """
        Parses and saves field data in fields_list for future reference

        :param model: the name of the current model
        :param fields: the model fields to be parsed
        :param kwargs:
        :return:
        """

        for field in fields:
            try:
                field_name, field_type = self.__handle_tokens(field)

                if field_type is not None and field_name is not None:
                    field_content = self.__handle_field(
                        field_type=field_type,
                        field_name=field_name,
                        model=model
                    )

                    self.fields_list.append(field_content)

            except TypeError:
                return

    def find_resource_in_scope(self, model):
        """
        Searches the current directory for the specified model
        either by filename or import statement in __init__.py

        If the model is not found in the current scope, a warning is displayed
        asking the user whether the model should be created.
        """

        path = f"{os.getcwd()}"

        init = f"{path}/__init__.py"

        filename = model.lower()

        if not model.endswith('.py'):
            filename += '.py'

        if filename in os.listdir(path):
            return

        try:
            for line in fileinput.input(init):
                if model.capitalize() in line:
                    fileinput.close()
                    return
        except FileNotFoundError:
            pass

        fileinput.close()

        if click.confirm(DEFAULT_NOT_IN_SCOPE_WARNING.format(model.capitalize(), self.app_name)):
            self.dependencies_list.append(model.capitalize())

    def handle_dependencies(self, app, **kwargs):

        if len(self.dependencies_list) < 1:
            return False

        for model in self.dependencies_list:
            template = 'model.tpl'
            template_import = 'model-import.tpl'

            table_name = f"{app}_{inflection.pluralize(model)}"

            self.default_create(
                model,
                templates_directory=TEMPLATE_DIR,
                template=template,
                template_import=template_import,
                context={
                    'model': model,
                    'abstract': False,
                    'fields': [],
                    'imports': [],
                    'db_table': table_name,
                }
            )

        return True

    def append_import(self, value, scope=''):
        if value not in self.imports_list:
            self.imports_list.append([f'{scope}.{value}', f'{inflection.camelize(value)}'])

    def append_special_import(self, value, scope=''):
        """
        Creates an import statement for special inheritance.

        :param scope:
        :param value:
        :return:
        """
        if value in DEFAULT_SPECIAL_INHERITABLE_TYPES:
            statement = DEFAULT_SPECIAL_INHERITABLE_TYPES[value]
            value = inflection.camelize(inflection.underscore(value))
            self.special_import = (statement, value)
            return True
        else:
            model = inflection.camelize(inflection.underscore(value))
            value = inflection.underscore(value)
            if scope:
                if self.project_name:
                    self.special_import = (f'{self.project_name}.{scope}.models.{value}', model)
                else:
                    self.special_import = (f'{scope}.models.{value}', model)
            else:
                self.special_import = (f'.{value}', model)
        return False



import os
import subprocess
from .runner import RunnerHelper

DEFAULT_RUNSERVER_COMMAND = ['python3', 'manage.py', 'runserver', '8000']

DEFAULT_PROCESS_SUBGROUP = 'django-run-server-group-soliloquy'


class ServerHelper(RunnerHelper):

    def run(self, **kwargs):
        """
        Starts the default Django development server.
        """
        os.chdir(self.cwd)

        try:
            DEFAULT_RUNSERVER_COMMAND[-1] = str(kwargs['port']) \
                if kwargs['port'] \
                else DEFAULT_RUNSERVER_COMMAND[-1]
        except KeyError:
            pass

        subprocess.call(DEFAULT_RUNSERVER_COMMAND)



import os
import click
from .helpers import *
from django_clite.helpers.logger import log_error, log_standard
from django_clite.helpers import get_project_name
from django_clite.helpers import find_project_files


def not_an_app_directory_warning():
    if not ('apps.py' in os.listdir('.')):
        log_error("Not inside an app directory")
        raise click.Abort


def ensure_test_directory(cwd):
    if 'tests' in os.listdir(cwd):
        pass
    else:
        try:
            os.mkdir('tests')
        except FileExistsError:
            pass


@click.group()
@click.option('--dry', is_flag=True, help="Display output without creating files.")
@click.option('--force', is_flag=True, help="Override any conflicting files.")
@click.option('--verbose', is_flag=True, help="Run in verbose mode.")
@click.pass_context
def generate(ctx, dry, force, verbose):
    """
    Adds models, routes, and other resources
    """
    not_an_app_directory_warning()

    ctx.ensure_object(dict)

    p, m, f = find_project_files(os.getcwd())

    ctx.obj['dry'] = dry
    ctx.obj['force'] = force
    ctx.obj['verbose'] = verbose
    ctx.obj['in_app'] = 'apps.py' in os.listdir('.')
    ctx.obj['cwd'] = os.getcwd()
    ctx.obj['admin'] = f"{os.getcwd()}/admin/"
    ctx.obj['admin_inlines'] = f"{os.getcwd()}/admin/inlines/"
    ctx.obj['fixtures'] = f"{os.getcwd()}/fixtures/"
    ctx.obj['forms'] = f"{os.getcwd()}/forms/"
    ctx.obj['migrations'] = f"{os.getcwd()}/migrations/"
    ctx.obj['models'] = f"{os.getcwd()}/models/"
    ctx.obj['models_tests'] = f"{os.getcwd()}/models/tests/"
    ctx.obj['managers'] = f"{os.getcwd()}/models/managers"
    ctx.obj['serializers'] = f"{os.getcwd()}/serializers/"
    ctx.obj['serializers_tests'] = f"{os.getcwd()}/serializers/tests/"
    ctx.obj['tests'] = f"{os.getcwd()}/tests/"
    ctx.obj['templates'] = f"{os.getcwd()}/templates/"
    ctx.obj['views'] = f"{os.getcwd()}/views/"
    ctx.obj['viewsets'] = f"{os.getcwd()}/viewsets/"

    if f is not None:
        ctx.obj['project_name'] = get_project_name(f)
    else:
        ctx.obj['project_name'] = None


@generate.command()
@click.argument('name')
@click.option('--inline', is_flag=True, help='Register admin model as inline.')
@click.option('--stub-permissions', is_flag=True, help='Add permission stubs to admin model.')
@click.argument("fields", nargs=-1, required=False)
@click.pass_context
def admin(ctx, name, inline, fields, stub_permissions):
    """
    Generates an admin model within the admin package.
    """

    path = ctx.obj['admin']

    fields = [f for f in fields]

    if inline:
        path = ctx.obj['admin_inlines']

    helper = AdminHelper(
        cwd=path,
        dry=ctx.obj['dry'],
        force=ctx.obj['force'],
        verbose=ctx.obj['verbose']
    )

    helper.create(model=name, fields=fields, inline=inline, permissions=stub_permissions)


@generate.command()
@click.argument('name')
@click.argument("fields", nargs=-1, required=False)
@click.option('-n', '--number', default=1, help='Number of objects to create in fixture.')
@click.pass_context
def fixture(ctx, name, fields, number):
    """
    Generates model fixtures.
    """

    path = ctx.obj.get('fixtures')

    helper = FixtureHelper(
        cwd=path,
        dry=ctx.obj.get('dry'),
        force=ctx.obj.get('force'),
        verbose=ctx.obj.get('verbose')
    )

    helper.create(model=name, fields=fields, total=number)


@generate.command()
@click.argument("name", required=True)
@click.pass_context
def form(ctx, name):
    """
    Generates a model form within the forms package.
    """

    path = ctx.obj['forms']

    helper = FormHelper(
        cwd=path,
        dry=ctx.obj['dry'],
        force=ctx.obj['force'],
        verbose=ctx.obj['verbose']
    )

    helper.create(model=name)


@generate.command()
@click.argument("name", required=True)
@click.pass_context
def manager(ctx, name):
    """
    Generates a model manager under the model managers directory.
    """
    path = ctx.obj['managers']

    helper = ManagerHelper(
        cwd=path,
        dry=ctx.obj['dry'],
        force=ctx.obj['force'],
        verbose=ctx.obj['verbose']
    )

    helper.create(model=name)


@generate.command()
@click.option('-a', '--abstract', is_flag=True, help="Creates an abstract model type.")
@click.option('-t', '--test-case', is_flag=True, help="Creates a TestCase for model.")
@click.option('-f', '--full', is_flag=True, help="Adds all related resources and TestCase")
@click.option('--register-admin', is_flag=True, help="Register model to admin site.")
@click.option('--register-inline', is_flag=True, help="Register model to admin site as inline.")
@click.option('-m', '--is-managed', is_flag=True, help="Add created_by and updated_by fields.")
@click.option('-i', '--inherits', '--extends', required=False, help="Add model inheritance.")
@click.option('--app', required=False, help="If base model inherits is in another app.")
@click.option('--api', is_flag=True, help='Only add api-related files.')
@click.argument("name", required=True)
@click.argument("fields", nargs=-1, required=False)
@click.pass_context
def model(ctx, name, full, abstract, fields, register_admin,
          register_inline, test_case, inherits, api, app, is_managed):
    """
    Generates a model under the models directory.
    One can specify multiple attributes after the model's name, like so:

        D g model track int:number char:title fk:album bool:is_favorite

    This will generate a Track model and add a foreign key of Album.
    If the model is to be added to admin.site one can optionally opt in by specifying the --register-admin flag.
    """

    # Ensure --app is used only if --inherits is used
    if app and not inherits:
        log_error("You've specified an app inheritance scope but did not specify the model to inherit from.")
        log_error("Please rerun the command like so:")
        log_standard(f"D generate model {name} --inherits BASE_MODEL --app {app}")
        raise click.Abort

    path = ctx.obj['models']

    helper = ModelHelper(
        cwd=path,
        dry=ctx.obj['dry'],
        force=ctx.obj['force'],
        verbose=ctx.obj['verbose']
    )

    ensure_test_directory(path)

    model_fields = helper.create(
        model=name,
        api=api,
        abstract=abstract,
        fields=fields,
        inherits=inherits,
        scope=app,
        project=ctx.obj['project_name'],
        is_managed=is_managed,
    )

    if api:
        ctx.invoke(test, name=name, scope="model")
        ctx.invoke(serializer, name=name)
        ctx.invoke(viewset, name=name)

    if register_admin or full:
        ctx.invoke(admin, name=name, fields=model_fields)

    if register_inline or full:
        ctx.invoke(admin, name=name, inline=True)

    if (test_case or full) and not api:
        ctx.invoke(test, name=name, scope="model")

    if full and not api:
        ctx.invoke(serializer, name=name)
        ctx.invoke(viewset, name=name)

    if full:
        ctx.invoke(form, name=name)
        ctx.invoke(template, name=name)
        ctx.invoke(view, name=name, class_type="list")
        ctx.invoke(view, name=name, class_type="detail")

    # Retuning model fields
    return model_fields


@generate.command()
@click.argument("name", required=True)
@click.argument("fields", nargs=-1)
@click.option('-i', '--inherits', '--extends', required=False, help="Add model inheritance.")
@click.option('-m', '--is-managed', is_flag=True, help="Add created_by and updated_by fields.")
@click.option('--api', is_flag=True, help='Only add api-related files.')
@click.pass_context
def resource(ctx, name, fields, inherits, api, is_managed):
    """
    Generates an app resource.

    This is ideal to add a model along with admin, serializer, view, viewset, template, and tests.
    You can invoke this command the same way you would the model command:

        D g resource track int:number char:title fk:album bool:is_featured

    This will generate a model with the specified attributes and all the related modules specified above.

    In case you're building an api, and don't need forms, templates and views, you can pass the --api flag to the command
    in order to prevent these files from being created.
    """

    ctx.invoke(
        model,
        name=name,
        api=api,
        register_admin=True,
        register_inline=True,
        fields=fields,
        test_case=True,
        inherits=inherits,
        is_managed=is_managed,
    )

    ctx.invoke(serializer, name=name)

    ctx.invoke(viewset, name=name)

    if not api:
        ctx.invoke(form, name=name)
        ctx.invoke(template, name=name)
        ctx.invoke(view, name=name, class_type='list', no_template=True)


@generate.command()
@click.argument("name", required=True)
@click.pass_context
def serializer(ctx, name):
    """
    Generates a serializer for a given model.

    Checks for the existence of the specified model in models.py
    before attempting to create a serializer for it. Aborts if model is not found.
    """

    path = ctx.obj['serializers']

    helper = SerializerHelper(
        cwd=path,
        dry=ctx.obj['dry'],
        force=ctx.obj['force'],
        verbose=ctx.obj['verbose']
    )

    helper.create(model=name)

    ensure_test_directory(path)

    ctx.invoke(test, name=name, scope='serializer')


@generate.command()
@click.argument("name", required=True)
@click.option("-c", "--class-type", type=click.Choice(['create', 'detail', 'list', 'edit', 'update']))
@click.pass_context
def template(ctx, name, class_type):
    """
    Generates an html template.
    """

    path = ctx.obj['templates']

    helper = TemplateHelper(
        cwd=path,
        dry=ctx.obj['dry'],
        force=ctx.obj['force'],
        verbose=ctx.obj['verbose']
    )

    helper.create(model=name, class_type=class_type)


@generate.command()
@click.argument("name", required=True)
@click.option("-s", "--scope", type=click.Choice(['model', 'serializer']), required=True)
@click.pass_context
def test(ctx, name, scope):
    """
    Generates a new TestCase.
    """

    # TODO: Find better way to deal with scope
    path = ctx.obj[f'{scope}s_tests']

    helper = TestHelper(
        cwd=path,
        dry=ctx.obj['dry'],
        force=ctx.obj['force'],
        verbose=ctx.obj['verbose']
    )

    ensure_test_directory(ctx.obj[f'{scope}s'])

    helper.create(model=name, scope=scope)


@generate.command()
@click.argument("name", required=True)
@click.option("-c", "--class-type", type=click.Choice(['create', 'detail', 'list', 'edit', 'update']))
@click.option('--no-template', is_flag=True, default=False, help='Generate related template.')
@click.pass_context
def view(ctx, name, class_type, no_template):
    """
    Generates a view function or class.
    """

    path = ctx.obj['views']

    helper = ViewHelper(
        cwd=path,
        dry=ctx.obj['dry'],
        force=ctx.obj['force'],
        verbose=ctx.obj['verbose']
    )

    class_type = 'form' if class_type == 'edit' else class_type

    helper.create(model=name, class_type=class_type)

    if no_template:
        pass
    else:
        ctx.invoke(template, name=name, class_type=class_type)


@generate.command()
@click.option('-r', '--read-only', is_flag=True, help="Create a read-only viewset.")
@click.argument("name", required=True)
@click.pass_context
def viewset(ctx, read_only, name):
    """
    Generates a viewset for a serializable model.
    """

    path = ctx.obj['viewsets']

    helper = ViewSetHelper(
        cwd=path,
        dry=ctx.obj['dry'],
        force=ctx.obj['force'],
        verbose=ctx.obj['verbose']
    )

    helper.create(
        model=name,
        read_only=read_only
    )


# @generate.command()
@click.option('-s', '--source', is_flag=True, help="Base table for SQL view.")
@click.argument("name", required=True)
@click.argument("fields", nargs=-1, required=False)
@click.pass_context
def sql_view(ctx, name, fields, source):
    """
    Generates a model as an SQL view.
    One can specify multiple attributes after the model's name, like so:

        D g sql_view track int:number char:title --source app_tracks
    """

    path = ctx.obj['models']

    helper = ModelHelper(
        cwd=path,
        dry=ctx.obj['dry'],
        force=ctx.obj['force'],
        verbose=ctx.obj['verbose']
    )

    helper.create(
        model=name,
        is_sql=True,
        source=source,
        abstract=False,
        inherits=None,
        fields=fields,
    )



import os
import click
from django_clite.helpers.logger import log_error
from django_clite.helpers import find_project_files
from django_clite.helpers import get_project_name
from django_clite.helpers import save_to_settings
from .helpers import (
    BuildHelper,
    EnvironmentHelper,
    MigrationHelper,
    ServerHelper,
    DockerHelper
)

DEFAULT_MISSING_ARGS_ERROR = 'Missing arguments: {}.'

DEFAULT_TOO_MANY_ARGS_ERROR = 'Too many arguments passed.'


def not_an_app_directory_warning():
    if not ('apps.py' in os.listdir('.')):
        log_error("Not inside an app directory")
        raise click.Abort


def not_in_project_warning():
    path, management, file = find_project_files(os.getcwd())
    if not management:
        log_error('Cannot find manage.py in directory')
        raise click.Abort


@click.group()
@click.pass_context
def run(ctx):
    """
    Run maintenance, development, and deployment scripts.
    """

    not_in_project_warning()

    ctx.ensure_object(dict)

    p, m, f = find_project_files(os.getcwd())

    ctx.obj['path'] = p
    ctx.obj['management'] = m
    ctx.obj['file'] = f
    ctx.obj['project_name'] = get_project_name(f)


# @run.command()
@click.pass_context
def build(ctx):
    """
    Run migrations and collect static files.
    """

    path = ctx.obj['management']
    BuildHelper.start(path)


# @run.command()
@click.pass_context
def deploy(ctx):
    """
    Push your project to your deployment server.

    This will prepare your project for deployment and run some checks for you. For example, the CLI will check if
    DEBUG or SECRET_KEY are properly configured. It will check if you have pending migrations and also collect
    staticfiles to your specified staticfiles location.

    First define your deployment upstream so the CLI can push your code to it. Once that is defined,
    in .config/upstream, the CLI will use the upstream server information to deploy your code.

    Supported deployment upstreams:
    - Dokku

    \b
        Define your upstream as DOKKU and add a DOKKU_HOST in your upstream configuration.
        The host needs to be an IP or fully qualified domain name (FQDN) for your upstream server.

    - Bitbucket, Github, and Gitlab

    \b
        Define your upstream as GIT and if your .git repository has an `origin` remote,
        the CLI will push your code to that server.
    """
    pass


@run.command()
@click.pass_context
@click.option('-f', '--filepath', type=click.Path(exists=True), required=False, help='Path to environment file.')
@click.option('--no-dokku', is_flag=True, help='Skip dokku export.')
@click.option('--no-example', is_flag=True, help='Skip example export.')
def export_env(ctx, filepath, no_dokku, no_example):
    """
    Export environment variables.
    Use this command to export environment variables to an example file or a dokku config file.
    For example environment file, all values are striped out, only keys are exported.

    \b
    In .env-dokku file:
    dokku config:set --no-restart PROJECT_NAME VARIABLE1=value1
    dokku config:set --no-restart PROJECT_NAME VARIABLE2=value2

    \b
    In .env-example file:
    VARIABLE1=
    VARIABLE2=

    The CLI assumes that your environment file lives next to the management file (manage.py).
    If that is not the case for your project, your can specify the path for the environment file
    (or just its name if in current directory) by passing the -f, --filepath option:

    \b
    D run export-env -f [filepath]
    """

    helper = EnvironmentHelper(cwd=ctx.obj['management'])

    helper.run(
        path=ctx.obj['management'],
        project_name=ctx.obj['project_name'],
        filepath=filepath,
        destination=ctx.obj['management'],
        no_dokku=no_dokku,
        no_example=no_example
    )


# @run.command()
@click.option('-a', '--app', type=str, required=True, help='The app whose fixtures we must load.')
@click.option('-r', '--recursive', is_flag=True, help='Load all fixtures in directory.')
@click.argument('fixture', required=False)
@click.pass_context
def load_data(ctx, app, recursive, fixture):
    """
    Load data from fixtures.

    Loads all data contained in either .yaml or .json file within the fixtures directory of the specified app. For example:

    \b
        D run load-data --app blog articles
    
    This will load the articles.json or articles.yaml fixtures into the database.

    By specifying the --recursive flag, one can load all fixtures contained within the app's fixtures directory.
    """

    if recursive and fixture:
        log_error(DEFAULT_TOO_MANY_ARGS_ERROR)
        raise click.Abort
    elif not recursive and not fixture:
        log_error(DEFAULT_MISSING_ARGS_ERROR.format('--fixture or --recursive'))
        raise click.Abort

    path = ctx.obj['path']
    management = ctx.obj['management']
    
    print(path)
    print(management)


@run.command()
@click.option('-a', '--app', type=str, required=False)
@click.argument('options', nargs=-1, required=False)
@click.pass_context
def migrations(ctx, app, options):
    """
    Run database migrations.

    This combines both `makemigrations` and `migrate` commands into one. For example:

        D run migrations blog
    
    will accomplish the same as the following two commands:

    \b
        ./manage.py makemigrations blog && \\
        ./manage.py migrate blog

    Another thing this command seeks to accomplish is to bypass the need to navigate to
    the top of the directory in order to have access to the `manage.py` module. As long
    as the command is ran from within one of the following scopes, the command will work as intended:

    \b
        /project
        /project/project
        /project/project/app

    """

    path = ctx.obj['management']

    MigrationHelper.run(path, app, options)


@run.command()
@click.option('-p', '--port', type=int, required=False, help='The port the server will listen on.')
@click.pass_context
def server(ctx, port):
    """
    Runs the development server.
    """
    helper = ServerHelper(cwd=ctx.obj['management'])

    helper.run(
        path=ctx.obj['management'],
        port=port
    )


# @run.command()
@click.option('-k', '--key', required=True, help='Parameter name.')
@click.option('-v', '--value', required=True, help='Parameter value.')
@click.pass_context
def update_setting(ctx, key, value):
    """
    Update project settings.
    """

    save_to_settings(
        value=f"{value}",
        parameter=key,
        path=ctx.obj['path']
    )


#######################################

@click.group()
@click.option('--create-config', is_flag=True, help='Create Dockerfile and docker-compose.yml')
@click.option('--verbose', is_flag=True, help='Run in verbose mode.')
@click.pass_context
def docker(ctx, verbose, create_config):
    """
    Run Docker-related options for your project.
    """

    ctx.obj['verbose'] = verbose

    ctx.obj['docker'] = DockerHelper(cwd=ctx.obj['management'])

    if create_config:
        ctx.invoke(create_dockerfile)
        ctx.invoke(create_compose)


@docker.command()
@click.pass_context
def build(ctx):
    """
    Build Docker container for this project.
    """

    return ctx.obj['docker'].build(verbose=ctx.obj['verbose'])


@docker.command()
@click.pass_context
def start(ctx):
    """
    Start Docker container for this project.
    """

    return ctx.obj['docker'].start(verbose=ctx.obj['verbose'])


@docker.command()
@click.pass_context
def create_dockerfile(ctx):
    """
    Creates a Dockerfile for this project.
    """

    return ctx.obj['docker'].create_dockerfile(
        project=ctx.obj['project_name']
    )


@docker.command()
@click.pass_context
def create_compose(ctx):
    """
    Creates a docker-compose file for this project.
    """

    return ctx.obj['docker'].create_compose(
        project=ctx.obj['project_name']
    )


run.add_command(docker)



import click

DEFAULT_IMPORT_WARNING = "Module already imported. Skipping..."

DEFAULT_OVERRIDE_WARNING = "File {} already exists. Override file?"

DEFAULT_NOUN_NUMBER_WARNING = "Tip: resource name should be singular."

DEFAULT_NOUN_NUMBER_OPTION = "Change resource name from {} to {}?"

DEFAULT_PARSED_CONTENT_LOG = """Filename: {}\nFilepath: {}\n\n---- Begin content ----\n{}\n---- End content ----\n\n"""

DEFAULT_DESTROY_LOG = """Will delete...\nFilename: {}\nFilepath: {}\n\nWill also remove imports in __init__.py"""

DEFAULT_APP_CREATION_LOG = """Successfully created application: {}"""

DEFAULT_ERRORS = {
    "project": "Unable to create project. Will exit.",
    "app": "Unable to create app: {}. Skipping...",
    "package": "Unable to create package. Will exit.",
    "folder": "Unable to create folder. Skipping...",
    "repo": "Unable to initialize repository. Skipping...",
    "touch": "Unable to create default files. Skipping...",
    "clean": "Unable to remove default files. Skipping...",
}

DEFAULT_MANAGEMENT_ERROR = """The CLI could not decipher your django project.
Please try to create your app inside a Django project directory."""

DEFAULT_MANAGEMENT_ERROR_HELP = """If your project uses custom configuration, you can pass some context to the CLI:
    Specify the manage.py directory with the flag: --directory/-d
    Specify the project name with the flag: --project/-p
    
    Example:
    D create app classroom --project school --directory path/to/management
"""

DEFAULT_MANAGEMENT_TIP = """Tip:
If your project is called `website`, for example, 
you need to run `D create app app_name` from within `website` or `website/website`.
"""

DEFAULT_CREATE_MESSAGE = 'Successfully created {} for {}.'

DEFAULT_DELETE_MESSAGE = 'Successfully deleted {} for {}.'


def log_error(message, **kwargs):
    click.secho(message=message, **kwargs, fg='red')


def log_info(message, **kwargs):
    click.secho(message=message, **kwargs, fg='blue')


def log_success(message, **kwargs):
    click.secho(message=message, **kwargs, fg='green')


def log_standard(message, **kwargs):
    click.secho(message=message, **kwargs)



import inflection


def sanitized_string(text):
    """
    Ensures strings are properly sanitized
    and no special characters are present.
    """

    r = inflection.transliterate(text)
    r = r.replace(' ', '-')
    return inflection.underscore(r).lower()



import os
import click
from .helpers import *
from django_clite.helpers.logger import log_error


def not_an_app_directory_warning():
    if not ('apps.py' in os.listdir('.')):
        log_error("Not inside an app directory")
        raise click.Abort


def ensure_test_directory(cwd):
    if 'tests' in os.listdir(cwd):
        pass
    else:
        try:
            os.mkdir('tests')
        except FileExistsError:
            pass


@click.group()
@click.option('--dry', '--dry-run', is_flag=True, help="Display output without deleting files")
@click.option('--force', is_flag=True, help="Override any conflicting files.")
@click.option('--verbose', is_flag=True, help="Run in verbose mode.")
@click.pass_context
def destroy(ctx, dry, force, verbose):
    """
    Removes models, serializers, and other resources
    """
    if not dry:
        not_an_app_directory_warning()

    ctx.ensure_object(dict)
    ctx.obj['dry'] = dry
    ctx.obj['force'] = force
    ctx.obj['verbose'] = verbose
    ctx.obj['in_app'] = 'apps.py' in os.listdir('.')
    ctx.obj['cwd'] = os.getcwd()
    ctx.obj['admin'] = f"{os.getcwd()}/admin/"
    ctx.obj['admin_inlines'] = f"{os.getcwd()}/admin/inlines/"
    ctx.obj['fixtures'] = f"{os.getcwd()}/fixtures/"
    ctx.obj['forms'] = f"{os.getcwd()}/forms/"
    ctx.obj['models'] = f"{os.getcwd()}/models/"
    ctx.obj['models_tests'] = f"{os.getcwd()}/models/tests/"
    ctx.obj['managers'] = f"{os.getcwd()}/models/managers/"
    ctx.obj['serializers'] = f"{os.getcwd()}/serializers/"
    ctx.obj['serializers_tests'] = f"{os.getcwd()}/serializers/tests/"
    ctx.obj['tests'] = f"{os.getcwd()}/tests/"
    ctx.obj['templates'] = f"{os.getcwd()}/templates/"
    ctx.obj['views'] = f"{os.getcwd()}/views/"
    ctx.obj['viewsets'] = f"{os.getcwd()}/viewsets/"


@destroy.command()
@click.argument('name')
@click.option('--inline', is_flag=True, help='Destroy inline admin model')
@click.pass_context
def admin(ctx, name, inline):
    """
    Destroys an admin model or inline.
    """

    path = ctx.obj['admin']

    if inline:
        path = ctx.obj['admin_inlines']

    h = AdminHelper(
        cwd=path,
        dry=ctx.obj['dry'],
        force=ctx.obj['force'],
        verbose=ctx.obj['verbose']
    )

    h.delete(model=name, inline=inline)


@destroy.command()
@click.argument('name')
@click.pass_context
def fixture(ctx, name):
    """
    Destroys a fixture.
    """

    path = ctx.obj.get('fixtures')

    helper = FixtureHelper(
        cwd=path,
        dry=ctx.obj['dry'],
        force=ctx.obj['force'],
        verbose=ctx.obj['verbose']
    )

    helper.delete(model=name)


@destroy.command()
@click.argument('name', required=True)
@click.pass_context
def form(ctx, name):
    """
    Destroys a form.
    """

    path = ctx.obj['forms']

    h = FormHelper(
        cwd=path,
        dry=ctx.obj['dry'],
        force=ctx.obj['force'],
        verbose=ctx.obj['verbose']
    )

    h.delete(model=name)


@destroy.command()
@click.argument('name', required=True)
@click.pass_context
def manager(ctx, name):
    """
    Destroys a model manager.
    """

    path = ctx.obj['managers']

    h = ManagerHelper(
        cwd=path,
        dry=ctx.obj['dry'],
        force=ctx.obj['force'],
        verbose=ctx.obj['verbose']
    )

    h.delete(model=name)


@destroy.command()
@click.argument('name')
@click.option('--unregister-admin', is_flag=True, help="Unregister model from the admin site.")
@click.option('--unregister-inline', is_flag=True, help="Unregister inline model from the admin site.")
@click.option('--test-case', is_flag=True, help="Delete TestCases for model.")
@click.option('--full', is_flag=True, help="Delete admin, inline, and TestCase")
@click.pass_context
def model(ctx, name, full, unregister_admin, unregister_inline, test_case):
    """
    Destroys a model.
    """

    path = ctx.obj['models']

    h = ModelHelper(
        cwd=path,
        dry=ctx.obj['dry'],
        force=ctx.obj['force'],
        verbose=ctx.obj['verbose']
    )

    ensure_test_directory(path)

    h.delete(model=name)

    if unregister_admin or full:
        ctx.invoke(admin, name=name)

    if unregister_inline or full:
        ctx.invoke(admin, name=name, inline=True)

    if test_case or full:
        ctx.invoke(test, name=name, scope='model')

    if full:
        ctx.invoke(form, name=name)
        ctx.invoke(serializer, name=name)
        ctx.invoke(test, name=name, scope='serializer')
        ctx.invoke(template, name=name)
        ctx.invoke(view, name=name, class_type="list")
        ctx.invoke(view, name=name, class_type="detail")
        ctx.invoke(viewset, name=name)


@destroy.command()
@click.argument('name')
@click.pass_context
def resource(ctx, name):
    """
    Destroys a resource and its related modules.
    """

    ctx.invoke(
        model,
        name=name,
        unregister_admin=True,
        unregister_inline=True,
        test_case=True
    )

    ctx.invoke(serializer, name=name)

    ctx.invoke(viewset, name=name)

    ctx.invoke(form, name=name)

    ctx.invoke(template, name=name)

    if not ctx.invoke(view, name=name, class_type='list'):
        ctx.invoke(view, name=name, class_type='detail')
        ctx.invoke(view, name=name)


@destroy.command()
@click.argument('name')
@click.pass_context
def serializer(ctx, name):
    """
    Destroys a serializer.
    """

    path = ctx.obj['serializers']

    h = SerializerHelper(
        cwd=path,
        dry=ctx.obj['dry'],
        force=ctx.obj['force'],
        verbose=ctx.obj['verbose']
    )

    h.delete(model=name)
    
    ensure_test_directory(path)

    ctx.invoke(test, name=name, scope='serializer')


@destroy.command()
@click.argument('name')
@click.option("-c", "--class-type", type=click.Choice(['list', 'detail']))
@click.pass_context
def view(ctx, name, class_type):
    """
    Destroys a view.
    """

    path = ctx.obj['views']

    h = ViewHelper(
        cwd=path,
        dry=ctx.obj['dry'],
        force=ctx.obj['force'],
        verbose=ctx.obj['verbose']
    )

    h.delete(model=name, class_type=class_type)


@destroy.command()
@click.argument('name')
@click.pass_context
def viewset(ctx, name):
    """
    Destroys a viewset.
    """

    path = ctx.obj['viewsets']

    h = ViewSetHelper(
        cwd=path,
        dry=ctx.obj['dry'],
        force=ctx.obj['force'],
        verbose=ctx.obj['verbose']
    )

    h.delete(model=name)


@destroy.command()
@click.argument('name')
@click.pass_context
def template(ctx, name):
    """
    Destroys a template.
    """

    path = ctx.obj['templates']

    h = TemplateHelper(
        cwd=path,
        dry=ctx.obj['dry'],
        force=ctx.obj['force'],
        verbose=ctx.obj['verbose']
    )

    h.delete(model=name)


@destroy.command()
@click.argument('name', required=True)
@click.option("-s", "--scope", type=click.Choice(['model', 'serializer']), required=True)
@click.pass_context
def test(ctx, name, scope):
    """
    Destroys a TestCase.
    """

    path = ctx.obj[f'{scope}s_tests']

    h = TestHelper(
        cwd=path,
        dry=ctx.obj['dry'],
        force=ctx.obj['force'],
        verbose=ctx.obj['verbose']
    )

    ensure_test_directory(ctx.obj[f'{scope}s'])

    h.delete(model=name, scope=scope)



# commands:runner
from .main import run
from .main import build
from .main import deploy
from .main import docker
from .main import export_env
from .main import load_data
from .main import migrations
from .main import server
from .main import update_setting



import os
import inflection
from django_clite.helpers.logger import *
from django_clite.helpers import sanitized_string
from django_clite.helpers import rendered_file_template
from django_clite.helpers import FSHelper

BASE_DIR = os.path.dirname(os.path.abspath(__file__)).rsplit('/', 1)[0]

TEMPLATE_DIR = os.path.join(BASE_DIR, 'templates')

TEMPLATES = [f for f in os.listdir(TEMPLATE_DIR) if f.endswith('tpl')]


class ViewSetHelper(FSHelper):

    def create(self, model, read_only=False):

        template = 'viewset.tpl'
        template_import = 'viewset-import.tpl'

        # TODO: Ensure serializer already exists

        self.default_create(
            model,
            templates_directory=TEMPLATE_DIR,
            template=template,
            template_import=template_import,
            scope='ViewSet',
            context={
                'model': model,
                'read_only': read_only,
                'route': inflection.pluralize(model),
            }
        )

    def delete(self, model):
        model = self.check_noun(model)
        classname = inflection.camelize(model)

        filename = f"{model}.py"
        template_import = 'viewset-import.tpl'

        if self.default_destroy_file(
            model=model,
            templates_directory=TEMPLATE_DIR,
            template_import=template_import
        ):

            resource = f"{classname}ViewSet"
            log_success(DEFAULT_DELETE_MESSAGE.format(filename, resource))



import os
import re
import sys
from os import path
import fileinput
from .logger import log_info


def walk_up(directory_path):
    """
    Implementation by Michele Pasin
    https://gist.github.com/zdavkeos/1098474#gistcomment-2943865

    Mimic os.walk, but walk 'up' instead of down the directory tree.
    """

    directory_path = path.realpath(directory_path)

    # get files in current dir
    try:
        names = os.listdir(directory_path)
    except Exception as e:
        print(e)
        return

    dirs, non_dirs = [], []
    for name in names:
        if path.isdir(path.join(directory_path, name)):
            dirs.append(name)
        else:
            non_dirs.append(name)

    yield directory_path, dirs, non_dirs

    new_path = path.realpath(path.join(directory_path, '..'))

    # see if we are at the top
    if new_path == directory_path:
        return

    for x in walk_up(new_path):
        yield x


def find_project_files(cwd):
    """
    Searches the current working directory for any of 4 key Django files:
    manage.py, settings.py, wsgi.py, and apps.py to determine from where a command
    is being run. This is used to ensure commands that create or delete resources
    are always run in the correct directory and within a Django project.
    """

    code = 0
    path = None
    project_directory = None
    management_file = None

    levels = (
        ('manage.py', 1),    # <-- project
        ('settings.py', 2),  # <-- project/project
        ('wsgi.py', 2),      # <-- project/project
        ('apps.py', 3),      # <-- project/project/app
    )

    for filename, c in levels:
        if filename in os.listdir(cwd):
            code = c
            path = cwd

    if code == 1:
        project_directory = path
        management_file = os.path.join(project_directory, 'manage.py')

    elif code == 2 or code == 3:
        for c, d, f in walk_up('.'):
            if 'manage.py' in f:
                project_directory = c
                management_file = os.path.join(project_directory, 'manage.py')
                break

    return path, project_directory, management_file


def find_settings_file(path):
    settings = os.path.join(path, 'settings.py')
    if not os.path.exists(settings):
        raise FileNotFoundError(
            "Can't find `settings.py` file: "f"{settings}"
        )
    return settings


def get_project_name(management_file=None, find_first=False):
    if find_first:
        p, m, f = find_project_files('.')
        management_file = f

    try:
        with open(management_file, 'r') as file:
            for line in file:
                if 'DJANGO_SETTINGS_MODULE' in line:
                    line = line.\
                        replace("'", "").\
                        replace(" ", "").\
                        replace('"', "").\
                        split(',')[-1].\
                        split('.')[0]
                    return line
    except TypeError:
        pass
    return None


def replace_line(value, parameter_name, settings_file):
    """
    Based on implementation from djecrety
    https://github.com/mrouhi13/djecrety/blob/master/djecrety/utils.py
    """

    parameter_is_exist = False
    if parameter_name:
        new_line = f'{parameter_name} = {value}'
        line_pattern = fr'^{parameter_name} = .*'

        for line in fileinput.input(settings_file, inplace=True):
            if re.match(line_pattern, line):
                parameter_is_exist = True
                line = re.sub(line_pattern, new_line, line)
            sys.stdout.write(line)

        if not parameter_is_exist:
            raise NameError(f"Can't find parameter name: {parameter_name}")
        return True
    return False


def save_to_settings(value, parameter, path):
    """
    Save the value to the given parameter in `settings.py` file.
    """
    settings = find_settings_file(path)

    return replace_line(value, parameter, settings)


def get_app_name(*args, **kwargs):
    """
    Searches current directory for apps.py in order to
    retrieve the application name from it.
    """

    # Assume cwd is *app/models
    # walk up from the directory in search for apps.py and AppConfig
    base = None

    if kwargs.get('verbose'):
        log_info(f"Searching for AppConfig at {os.getcwd()}")

    for root, dirs, files in walk_up(os.getcwd()):
        if "apps.py" in files:
            base = root

    if not base:
        if kwargs.get('verbose'):
            log_info(f"Cannot find for AppConfig for this app.")

        return "app"

    os.chdir(base)

    try:
        for line in fileinput.input('apps.py'):
            if "name = " in line:
                fileinput.close()
                return line.split(" = ")[-1] \
                    .lstrip() \
                    .replace("\n", "") \
                    .replace("'", "") \
                    .split('.')[-1]
        fileinput.close()
    except FileNotFoundError:
        pass

    os.chdir(PREVIOUS_WORKING_DIRECTORY)
    return "app"



import os
import inflection
from django_clite.helpers.logger import *
from django_clite.helpers import sanitized_string
from django_clite.helpers import rendered_file_template
from django_clite.helpers import FieldParser

BASE_DIR = os.path.dirname(os.path.abspath(__file__)).rsplit('/', 1)[0]

TEMPLATE_DIR = os.path.join(BASE_DIR, 'templates')

TEMPLATES = [f for f in os.listdir(TEMPLATE_DIR) if f.endswith('tpl')]


class FixtureHelper(FieldParser):

    def create(self, model, **kwargs):
        model = self.check_noun(model)
        model = sanitized_string(model)
        classname = inflection.camelize(model)

        if kwargs.get('fields') is not None:
            self.parse_fields(model, kwargs.get('fields'))

        template = 'fixture.tpl' \
            if not kwargs.get('template') \
            else kwargs.get('template')

        content = rendered_file_template(
            path=TEMPLATE_DIR,
            template=template,
            context={
                'total': int(kwargs.get('total')),
                'app': self.app_name,
                'classname': classname,
                'fields': self.fixture_fields,
            }
        )

        self.create_file(
            content=content,
            filename=f'{inflection.pluralize(model)}.json',
            path=self.cwd,
        )

    def delete(self, model, **kwargs):
        model = self.check_noun(model)
        classname = inflection.camelize(model)

        filename = f"{model}.json"

        if self.default_destroy_file(
            model=model,
            templates_directory=TEMPLATE_DIR
        ):

            resource = f"{classname} fixture"
            log_success(DEFAULT_DELETE_MESSAGE.format(filename, resource))



import os
import subprocess
from .runner import RunnerHelper
from django_clite.helpers.logger import log_error


class MigrationHelper(RunnerHelper):

    @classmethod
    def make_migrations(cls, path, app, options):
        os.chdir(path)

        try:
            management = os.path.join(path, 'manage.py')
            command = ['python3', management, 'makemigrations']
            if app is not None:
                command.append(app)
            # TODO: determine how to handle manage.py flags
            #   or whether we should support them, at all.
            # for option in options:
            #     command.append(option)
            subprocess.check_call(command)
        except subprocess.CalledProcessError:
            pass

    @classmethod
    def migrate(cls, path, app, options):
        os.chdir(path)

        try:
            management = os.path.join(path, 'manage.py')
            command = ['python3', management, 'migrate']
            if app is not None:
                command.append(app)
            # TODO: determine how to handle manage.py flags
            #   or whether we should support them, at all.
            # for option in options:
            #     command.append(option)
            subprocess.check_call(command)
        except subprocess.CalledProcessError:
            pass

    @classmethod
    def run(cls, path, app, options):
        if app:
            cls.make_migrations(path, app, options)
            cls.migrate(path, app, options=options)
        else:
            cls.make_migrations(path, app=None, options=options)
            cls.migrate(path, app=None, options=options)



from click.testing import CliRunner
from django_clite.commands.runner.main import create_settings


def test_create_settings():
    runner = CliRunner()
    result = runner.invoke(create_settings)
    assert result.exit_code == 0
